﻿using System;
using Chromium;
using System.IO;
using System.Net;
using System.Linq;
using System.Text;
using System.Data;
using System.Drawing;
using Chromium.Remote;
using NetDimension.NanUI;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Reflection;

namespace NanUIDemo
{
    public partial class Main : Formium
    {
        public Main() : base("http://res.app.local/Html/Main/index.html")
        {
            InitializeComponent();
            //页面加载完成时执行的代码
            //LoadHandler.OnLoadEnd += LoadHandler_OnLoadEnd;
            //设置最大化
            //Rectangle ScreenArea = Screen.GetWorkingArea(this);
            //Size = new Size(ScreenArea.Width, ScreenArea.Height);

            //温馨提示
            GlobalObject.AddFunction("showBox").Execute += (func, args) =>
            {
                this.RequireUIThread(() =>
                {
                    MessageBox.Show("你的电脑中毒了");
                });
            };

            //显示窗体
            GlobalObject.AddFunction("showWindow").Execute += (func, args) =>
            {
                this.RequireUIThread(() =>
                {
                    var main = new Main();
                    main.Show(this);
                });
            };

            //显示置顶窗体
            GlobalObject.AddFunction("showDialog").Execute += (func, args) =>
            {
                this.RequireUIThread(() =>
                {
                    var main = new Main();
                    main.ShowDialog(this);
                });
            };

            //开发者工具
            GlobalObject.AddFunction("ShowDevTools").Execute += (func, args) =>
            {
                this.RequireUIThread(() =>
                {
                    Chromium.ShowDevTools();//打开DevTools窗口观看JS控制台输出消息
                });
            };

            //C#执行JS代码(C#可传参数,但JS不返回参数)
            GlobalObject.AddFunction("CStoJS1").Execute += (func, args) =>
            {
                ExecuteJavascript("UTitle('" + "Demo" + "')");
            };

            //C#执行JS代码(C#可用传参数,JS返回参数(参数1:返回的数据集合,参数2:JS环境的异常对象))
            GlobalObject.AddFunction("CStoJS2").Execute += (func, args) =>
            {
                EvaluateJavascript("VTitle()", (value, exception) =>
                {
                    if (value.IsString)//判断返回的参数类型
                    {
                        var jsValue = value.StringValue;
                        MessageBox.Show(jsValue);
                    }
                });
            };

            //JS执行C#代码(JS可传参数,但C#不返回参数)
            GlobalObject.AddFunction("JStoCS1").Execute += (func, args) =>
            {
                var stringArgument = args.Arguments.FirstOrDefault(p => p.IsString);
                MessageBox.Show(stringArgument.StringValue);
            };

            //JS执行C#代码(传递多个参数)
            GlobalObject.AddFunction("JStoCS2").Execute += (func, args) =>
            {
                var jsArray = args.Arguments.FirstOrDefault(p => p.IsArray);
                for (int i = 0; i < jsArray.ArrayLength; i++)
                {
                    MessageBox.Show(jsArray.GetValue(i).StringValue);
                }
            };

            //JS执行C#代码(JS不传参数,C#返回参数)
            GlobalObject.AddFunction("JStoCS3").Execute += (func, args) =>
            {
                var friends = new string[] { "返回的参数1", "返回的参数2" };
                //创建CfrV8Value对象和此对象的Accssor
                var jsObjectAccessor = new CfrV8Accessor();
                var jsObject = CfrV8Value.CreateObject(jsObjectAccessor);
                //创建一个CfrV8Value数组
                var jsArray = CfrV8Value.CreateArray(friends.Length);
                for (int i = 0; i < friends.Length; i++)
                {
                    //把返回的参数写进去
                    jsArray.SetValue(i, CfrV8Value.CreateString(friends[i]));
                }
                //把返回的参数写到返回的数据中
                jsObject.SetValue("libName", CfrV8Value.CreateString("NanUI"), CfxV8PropertyAttribute.ReadOnly);
                jsObject.SetValue("friends", jsArray, CfxV8PropertyAttribute.DontDelete);
                args.SetReturnValue(jsObject);//返回参数
            };

            //JS执行C#代码(JS传参数,C#返回参数)
            GlobalObject.AddFunction("JStoCS4").Execute += (func, args) =>
            {
                var friends = new string[] { "返回的参数1", "返回的参数2" };
                var jsArray = args.Arguments.FirstOrDefault(p => p.IsArray);
                if (jsArray == null)//传递的参数是否为空
                {
                    jsArray = CfrV8Value.CreateArray(friends.Length);
                    for (int i = 0; i < friends.Length; i++)
                    {
                        jsArray.SetValue(i, CfrV8Value.CreateString(friends[i]));
                    }
                }
                else
                {
                    var newArray = CfrV8Value.CreateArray(jsArray.ArrayLength + friends.Length);

                    for (int i = 0; i < jsArray.ArrayLength; i++)
                    {
                        newArray.SetValue(i, jsArray.GetValue(i));
                    }

                    var jsArrayLength = jsArray.ArrayLength;

                    for (int i = 0; i < friends.Length; i++)
                    {
                        newArray.SetValue(i + jsArrayLength, CfrV8Value.CreateString(friends[i]));
                    }
                    jsArray = newArray;
                }
                args.SetReturnValue(jsArray);//返回参数
            };
        }
        private void LoadHandler_OnLoadEnd(object sender, Chromium.Event.CfxOnLoadEndEventArgs e)
        {
            //检查页面加载时是否为主框架
            if (e.Frame.IsMain)
            {
                Chromium.ShowDevTools();//打开DevTools窗口观看js控制台输出消息
            }
        }
    }
}